/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package clients.stockquote;

import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import org.apache.wsif.base.WSIFServiceImpl;
import org.apache.wsif.providers.soap.apacheaxis.WSIFDynamicProvider_ApacheAxis;
import org.apache.wsif.util.WSIFPluggableProviders;

import stockquote.wsifservice.StockquotePT;

/**
 * This sample shows how to use WSIF to invoke the addressbook service
 * using the static stub. It uses both the ports avaiable via the stub
 * to invoke the service.
 *
 * @author Sanjiva Weerawarana
 * @author Matthew J. Duftler
 * @author Alekander Slominski
 * @author Paul Fremantle
 * @author Nirmal Mukhi
 * @author Michael Beisiegel
 */
public class Main {

    private static void doit(StockquotePT service) throws Exception {
        String name1 = "IBM";
        System.err.println(">> Getting quote for '" + name1 + "'");
        float quote = service.getQuote(name1);
        System.err.println(">> Received quote: " + quote);
    }

    private static void usage() {
        System.err.println(
            "Usage: java "
                + Main.class.getName()
                + " portName wsdlLocation [soap|axis]");
        System.exit(1);
    }

    public static void main(String[] args) throws Exception {
        if (args.length > 3)
            usage();

        String protocol = null;
        String portName = args.length > 0 ? args[0] : null;
        if (args.length == 1 && (portName.equals("soap") || portName.equals("axis"))) {
            protocol = portName;
            portName = null;
        }

        String wsdlLocation = args.length > 1 ? args[1] : null;
        if (args.length == 2
            && (wsdlLocation.equals("soap") || wsdlLocation.equals("axis"))) {
            protocol = wsdlLocation;
            wsdlLocation = null;
        }

        protocol = args.length == 3 ? args[2] : "";
        if (!protocol.equals("")
            && !protocol.equals("soap")
            && !protocol.equals("axis"))
            usage();

        if (protocol.equals("axis"))
            WSIFPluggableProviders.overrideDefaultProvider(
                "http://schemas.xmlsoap.org/wsdl/soap/",
                new WSIFDynamicProvider_ApacheAxis());

        WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service = factory.getService(wsdlLocation, null, // serviceNS
        null, // serviceName
        "http://wsifservice.stockquote/", // portTypeNS
    "StockquotePT"); // portTypeName

        StockquotePT stub = null;
        if (portName != null) {
            System.err.println("\n\nUsing '" + portName + "' port:");
            stub = (StockquotePT) service.getStub(portName, StockquotePT.class);
            doit(stub);
        } else {
            System.err.println("\n\nUsing SOAP port:");
            stub = (StockquotePT) service.getStub("SOAPPort", StockquotePT.class);
            doit(stub);

            System.err.println("\n\nUsing Java port:");
            stub = (StockquotePT) service.getStub("JavaPort", StockquotePT.class);
            doit(stub);
        }

        if (protocol.equals("axis")) { // reset to default provider
            WSIFPluggableProviders.overrideDefaultProvider(
                "http://schemas.xmlsoap.org/wsdl/soap/",
                null);
        }
    }
}
