/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.jca.toolplugin;

/**
 * This interface allows Resource Adapter to provide a code generation contribution to the tool
 * environment. The tooling delegates to the implementation of this interface the generation
 * of all format handlers for the particular encoding and style. 
 * 
 * @author Hesham Fahmy <hfahmy@ca.ibm.com>
 * @author Piotr Przybylski <piotrp@ca.ibm.com>
 * @author Harlod Gartner
 */

public interface FormatHandlerGenerator
{
	public final static int XSD_TYPE =0;
	public final static int XSD_ELEMENT =1;

  /**
	* Generates a Custom Format Handler for a particular XSD Type or Element.
	* @param <code>generationPackage</code> The fully qualified package that the generator should use to generate its classes in.
	* @param <code>beanClass</code> The fully qualified name of the Java bean class that represents the XSD type (or element) for which a format handler is being generated.
	* @param <code>aDefinition</code> The WSDL definition from which the XSD type is derived.
	* @param <code>aBinding</code> The WSDL binding element that contains the type mapping for the XSD type that is being used.
	* @param <code>anEncoding</code> The <i>encoding</i> attribute off the TypeMapping element associated with the format handler to be generated.
	* @param <code>aStyle</code> The <i>style</i> attribute off the TypeMapping element associated with the format handler to be generated.
	* @param <code>xsdQname</code> The QName of the XSD element(or type) for which the format handler is being generated.
	* @param <code>elementType</code> The type of element represented by the <code>xsdQname</code> parameter. This is either <code>CustomFormatHandlerGenerator.XSD_TYPE</code>
	* 				or <code>CustomFormatHandlerGenerator.XSD_ELEMENT</code>. 
 	* @param <code>generationContext</code> The generation context or <code>null</code> is no context is required.
	* @return a <code>HashTable</code> that contains a set of  <code>java.io.InputStream</code> s. Each input stream contains the generated code for a particular
	* class generated by the format handler generator. The number of input streams returned depends on the number of classes that are generated by the format handler
	* generator.  The keys of the HashTable must be the name of each class (unqualified) that is represented by its corresponding InputStream value. The class name keys
	* are unqualified because they must all be in the <code>generationPackage</code> that is passed in as an input parameter.
	* @throws <code>org.apache.wsif.WSIFException</code> thrown if an error occurs during format handler generation. 
	*/

	public java.util.Hashtable generate (String generationPackage, String beanClass, javax.wsdl.Definition aDefinition,javax.wsdl.Binding aBinding, String anEncoding,
														String aStyle, javax.xml.namespace.QName xsdQname, int elementType, Object generationContext) throws org.apache.wsif.WSIFException;

	/**
	 * Creates a new context for format handler generation.
	 * The context is an optional object that can be used by a FormatHandlerGenerator to assist in code generation, or in keeping track of 
	 * artifacts generated. The details, and implementation, of the object are specific to each FormatHandlerGenerator.
	 * Tooling environments must call this method first, before invoking the FormatHandlerGenerator to generate code. The context that is returned must be used
	 * by the tooling environment for all subsequent calls to the FormatHandlerGenerator.
	 * @return A new generation context.
	 */

	public Object createGenerationContext();
}

