/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.jms;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.xml.namespace.QName;

import org.apache.wsif.format.jms.JMSFormatHandler;
import org.apache.wsif.logging.Trc;

public class PrimitiveTypeFormatHandler implements JMSFormatHandler {
	private static final long serialVersionUID = 1L;

    protected static DateFormat GREGORIAN_CALENDAR_DATE_FORMAT =
        new SimpleDateFormat("yyyy-MM-dd");
    protected static DateFormat STANDARD_DATE_FORMAT =
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    protected static DateFormat PRECISE_DATE_FORMAT =
        new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'.'SSS'Z'");

    QName name;
    Object primitiveType;
    Class type;
    
    /**
     * Constructor for CustomerInfoFormatHandler
     */
    public PrimitiveTypeFormatHandler(Class type) {
        super();
        this.type = type;
    }

    /**
     * @see JMSFormatHandler#getElement(String)
     */
    public Object getElement(String arg0) {
        return null;
    }

    /**
     * @see JMSFormatHandler#setElement(String, Object)
     */
    public void setElement(String arg0, Object arg1) {
    }

    /**
     * @see JMSFormatHandler#getElement(String, int)
     */
    public Object getElement(String arg0, int arg1) {
        return null;
    }

    /**
     * @see JMSFormatHandler#setElement(String, int, Object)
     */
    public void setElement(String arg0, int arg1, Object arg2) {
    }

    /**
     * @see JMSFormatHandler#getObjectPart()
     */
    public Object getObjectPart() {
        return primitiveType;
    }

    /**
     * @see JMSFormatHandler#getObjectPart(Class)
     */
    public Object getObjectPart(Class arg0) {
        return null;
    }

    public char getCharPart() {
        return 0;
    }

    public byte getBytePart() {
        return 0;
    }

    public short getShortPart() {
        return 0;
    }

    public int getIntPart() {
        return 0;
    }

    public long getLongPart() {
        return 0;
    }

    public float getFloatPart() {
        return 0;
    }

    public double getDoublePart() {
        return 0;
    }

	public QName getPartQName() {
		return name;
	}

    public void setObjectPart(Object arg0) {
        primitiveType = arg0;
    }

    public void setCharPart(char arg0) {
    }

    public void setBytePart(byte arg0) {
    }

    public void setShortPart(short arg0) {
    }

    public void setIntPart(int arg0) {
    }

    public void setLongPart(long arg0) {
    }

    public void setFloatPart(float arg0) {
    }

    public void setDoublePart(double arg0) {
    }

	public void setPartQName(QName name) {
		this.name = name;
	}

    /**
     * @see JMSFormatHandler#write(Message)
     */
    public void write(Message message) {
        Trc.entry(this, message);
        try {
            if (message instanceof javax.jms.TextMessage)
                write((javax.jms.TextMessage) message);
            else if (message instanceof javax.jms.ObjectMessage)
                write((javax.jms.ObjectMessage) message);
        } catch (Exception e) {
        	Trc.exception(e);
        }
        Trc.exit();
    }

    /**
     * @see JMSFormatHandler#read(Message)
     */
    public void read(Message message) {
        Trc.entry(this, message);
        try {
            if (message instanceof javax.jms.TextMessage)
                read((javax.jms.TextMessage) message);
            else if (message instanceof javax.jms.ObjectMessage)
                read((javax.jms.ObjectMessage) message);
        } catch (Exception e) {
        	Trc.exception(e);
        }
        Trc.exit();

    }

    private void write(javax.jms.TextMessage message) throws JMSException {
        String value = null;

        if (primitiveType == null)
            value = "";
        else if (primitiveType instanceof java.util.GregorianCalendar)
            value =
                GREGORIAN_CALENDAR_DATE_FORMAT.format(
                    ((java.util.GregorianCalendar) primitiveType).getTime());
        else if (primitiveType instanceof java.util.Date)
            value = STANDARD_DATE_FORMAT.format((java.util.Date) primitiveType);
        else if (primitiveType instanceof byte[])
            value = new String((byte[]) primitiveType);
        else {
            // Convert the primitiveType to a String
            value = primitiveType.toString();
        }

        if (value != null)
            message.setText(value);
    }

    private void write(javax.jms.ObjectMessage message) throws JMSException {
        if (primitiveType != null && primitiveType instanceof java.io.Serializable)
            message.setObject((java.io.Serializable) primitiveType);
    }

    private void read(javax.jms.TextMessage message) throws JMSException {
        String value = message.getText();

        Class cls = type;
        try {
            if (cls == String.class) {
                primitiveType = value;
            }
            // For byte array class
            else if (cls == byte[].class) {
                primitiveType = value.getBytes();
            }
            // For Gregorian Calendar
            else if (java.util.GregorianCalendar.class.isAssignableFrom(cls)) {
                java.util.GregorianCalendar result = new java.util.GregorianCalendar();
                result.setTime(GREGORIAN_CALENDAR_DATE_FORMAT.parse(value));
                primitiveType = result;
            }
            // For Date
            else if (java.util.Date.class.isAssignableFrom(cls)) {
                java.util.Date result = new java.util.Date();
                try {
                    result = STANDARD_DATE_FORMAT.parse(value);
                } catch (java.text.ParseException standardException) {
                	Trc.exception(standardException);
                    result = PRECISE_DATE_FORMAT.parse(value);
                }
                primitiveType = result;
            }
            // For all the rest
            else {
                java.lang.reflect.Constructor constructor =
                    cls.getConstructor(new Class[] { String.class });
                primitiveType = constructor.newInstance(new Object[] { value });
            }
        } catch (Exception e) {
        	Trc.exception(e);
            throw new JMSException("Unable to parse message");
        }

    }

    private void read(javax.jms.ObjectMessage message) throws JMSException {
        primitiveType = message.getObject();
        // Check to see if the contents is of the expected class type
        if (!primitiveType.getClass().isAssignableFrom(type))
            throw new JMSException("Unable to parse message");
    }
}