/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.wsdl.extensions.jms;

import javax.xml.namespace.QName;

import com.ibm.wsdl.Constants;

/**
 * WSDL Jms extension
 * 
 * @author <a href="mailto:ake@de.ibm.com">Hermann Akermann</a>
 * @author Ant Elder <antelder@apache.org>
 * @author Mark Whitlock <whitlock@apache.org>
 */
public class JMSConstants {

    // Namespace URIs. 
    public static final String NS_URI_JMS = "http://schemas.xmlsoap.org/wsdl/jms/";

    // Element names.
    public static final String ELEM_ADDRESS = "address";
    public static final String ELEM_PROPERTY = "property";
    public static final String ELEM_PROPERTY_VALUE = "propertyValue";
    public static final String ELEM_INPUT = "input";
    public static final String ELEM_OUTPUT = "output";
    public static final String ELEM_FAULT = "fault";
    public static final String ELEM_FAULT_INDICATOR = "faultIndicator";
    public static final String ELEM_FAULT_PROPERTY = "faultProperty";
    public static final String ATTR_MESSAGE_TYPE = "type";

    // managed environment elements
    public static final String ELEM_JMS_JMS_VENDOR_URI = "jmsVendorURI";
    public static final String ELEM_JMS_INIT_CXT_FACT = "initialContextFactory";
    public static final String ELEM_JMS_JNDI_PROV_URL = "jndiProviderURL";
    public static final String ELEM_JMS_DEST_STYLE = "destinationStyle";
    public static final String ELEM_JMS_JNDI_CONN_FACT_NAME =
        "jndiConnectionFactoryName";
        
    public static final String ELEM_JMS_JNDI_DEST_NAME = "jndiDestinationName";
    public static final String ELEM_JMS_JMS_PROV_DEST_NAME =
        "jmsProviderDestinationName";
        
    public static final String ELEM_JMS_JMS_IMPL_SPEC_URI =
        "jmsImplementationSpecificURI";
        
    public static final String ELEM_JMS_QUEUE = "queue";
    public static final String ELEM_JMS_TOPIC = "topic";

    // Jms Header, HeaderValue, Property, PropertyValue extension	
    public static final String ATTR_PART = "part";
    public static final String ATTR_NAME = "name";
    public static final String ATTR_TYPE = "type";
    public static final String ATTR_VALUE = "value";

    // Jms Header Field types
    public static final int HEADER_TYPE_JMSMESSAGEID =      0;
    public static final int HEADER_TYPE_JMSTIMESTAMP =     10;
    public static final int HEADER_TYPE_JMSCORRELATIONID = 20;
    public static final int HEADER_TYPE_JMSREPLYTO =       30;
    public static final int HEADER_TYPE_JMSDESTINATION =   40;
    public static final int HEADER_TYPE_JMSDELIVERYMODE =  50;
    public static final int HEADER_TYPE_JMSREDELIVERED =   60;
    public static final int HEADER_TYPE_JMSTYPE =          70;
    public static final int HEADER_TYPE_JMSEXPIRATION =    80;
    public static final int HEADER_TYPE_JMSPRIORITY =      90;

    // Jms Input/Output extension	
    public static final String ATTR_PARTS = "parts";
    public static final String ATTR_SCHEMA = "schema";

    // Qualified element names.
    public static final QName Q_ELEM_JMS_ADDRESS =
        new QName(NS_URI_JMS, ELEM_ADDRESS);
        
    public static final QName Q_ELEM_JMS_BINDING =
        new QName(NS_URI_JMS, Constants.ELEM_BINDING);
        
    public static final QName Q_ELEM_JMS_PROPERTY =
        new QName(NS_URI_JMS, ELEM_PROPERTY);
        
    public static final QName Q_ELEM_JMS_PROPERTY_VALUE =
        new QName(NS_URI_JMS, ELEM_PROPERTY_VALUE);
        
    public static final QName Q_ELEM_JMS_OPERATION =
        new QName(NS_URI_JMS, Constants.ELEM_OPERATION);
        
    public static final QName Q_ELEM_JMS_INPUT =
        new QName(NS_URI_JMS, ELEM_INPUT);
        
    public static final QName Q_ELEM_JMS_OUTPUT =
        new QName(NS_URI_JMS, ELEM_OUTPUT);

    public static final QName Q_ELEM_JMS_FAULT =
        new QName(NS_URI_JMS, ELEM_FAULT);

    public static final QName Q_ELEM_JMS_FAULT_INDICATOR =
        new QName(NS_URI_JMS, ELEM_FAULT_INDICATOR);

    public static final QName Q_ELEM_JMS_FAULT_PROPERTY =
        new QName(NS_URI_JMS, ELEM_FAULT_PROPERTY);

    // Jms message (body) types
    public static final int MESSAGE_TYPE_NOTSET = 0;
    public static final int MESSAGE_TYPE_BYTEMESSAGE = 10;
    public static final int MESSAGE_TYPE_MAPMESSAGE = 20;
    public static final int MESSAGE_TYPE_OBJECTMESSAGE = 30;
    public static final int MESSAGE_TYPE_STREAMMESSAGE = 40;
    public static final int MESSAGE_TYPE_TEXTMESSAGE = 50;
}