/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package async;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.apache.wsif.WSIFCorrelationId;
import org.apache.wsif.WSIFCorrelationService;
import org.apache.wsif.WSIFException;
import org.apache.wsif.util.WSIFCorrelationServiceLocator;
import org.apache.wsif.util.jms.WSIFJMSCorrelationId;

public class CorrelationServiceTest extends TestCase {

    public CorrelationServiceTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        return new TestSuite(CorrelationServiceTest.class);
    }

    public void setUp() {
    }

    public void testIt() {

        System.out.println("Testing the WSIFCorrelationService...");

        WSIFCorrelationService cs =
            WSIFCorrelationServiceLocator.getCorrelationService();

        WSIFCorrelationService cs2 =
            WSIFCorrelationServiceLocator.getCorrelationService();
        assertTrue(cs == cs2);

        try {

            WSIFCorrelationId cid = new WSIFJMSCorrelationId("1");
            cs.put(cid, "petra", (long) 0);

            cid = new WSIFJMSCorrelationId("2");
            cs.put(cid, "ant", (long) 0);

            cid = new WSIFJMSCorrelationId("3");
            cs2.put(cid, "tanya", 3000);

            cid = new WSIFJMSCorrelationId("1");
            String s = (String) cs.get(cid);
            assertTrue(s.equals("petra"));

            cid = new WSIFJMSCorrelationId("2");
            s = (String) cs.get(cid);
            assertTrue(s.equals("ant"));

            cid = new WSIFJMSCorrelationId("3");
            s = (String) cs.get(cid);
            assertTrue(s.equals("tanya"));

            System.out.println("\nwaiting for timeouts...");
            try {
                Thread.sleep(10000);
            } catch (Exception ex) {
                System.out.println("interupted early");
            }

            cid = new WSIFJMSCorrelationId("1");
            s = (String) cs.get(cid);
            assertTrue(s.equals("petra"));

            cid = new WSIFJMSCorrelationId("2");
            s = (String) cs.get(cid);
            assertTrue(s.equals("ant"));

            cid = new WSIFJMSCorrelationId("3");
            s = (String) cs.get(cid);
            assertTrue(s == null); // should have timed out

            cid = new WSIFJMSCorrelationId("2");
            cs.remove(cid);

            cid = new WSIFJMSCorrelationId("1");
            s = (String) cs.get(cid);
            assertTrue(s.equals("petra"));

            cid = new WSIFJMSCorrelationId("2");
            s = (String) cs.get(cid);
            assertTrue(s == null); // due to remove

			// test primative clasess
			Class[] clss =
				new Class[] {
					int.class,
					float.class,
					long.class,
					double.class,
					short.class,
					byte.class,
					boolean.class,
					void.class };
			Class[] clss2;
			cid = new WSIFJMSCorrelationId("P1");
			cs.put(cid, clss, (long) 0);
			clss2 = (Class[]) cs.get(cid);
			for (int i = 0; i < clss.length; i++) {
				assertTrue(
					"class " + clss[i] + " failed!!",
					clss[i] == clss2[i]);
			}
			
            System.out.println("WSIFCorrealtionService tests complete.");
        } catch (WSIFException ex) {
            ex.printStackTrace();
            assertTrue(false);
        }

    }

}
