/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package util;

import faults.FaultMsgTest;
import headers.HeadersTest;
import inout.InoutTest;
import interop.InteropDocTest;
import interop.InteropDocWrappedTest;
import invocation.DynamicInvokerTest;

import jms.JmsFaultTest;
import jms.JmsTest;
import jndi.JNDIAddressBookTest;
import junit.framework.Test;
import junit.framework.TestSuite;

import mime.MimeTest;
import providers.PlugableProvidersTest;
import providers.ProvidersInitialisationTest;
import serialization.SerializationTest;
import simpletypes.SOAPENCTest;
import soap.InputPartsTest;
import soap.MissingInputPartTest;
import soap.OutputPartsTest;
import soapinterop.InteropTest;
import stockquote.StockquoteTest;
import wsdl.WsdlLoadingTest;
import zipcode.ZIPCodeTest;

import addressbook.AddressBookTest;
import async.AsyncTests;
import chartype.CharTest;
import docStyle.MessagingAttachmentsTest;
import docStyle.NWBankTest;
import docStyle.ZipCodeAxisTest;

/**
 * Run JUnit tests on WSIF code.
 * Add new tests to the suite() method.
 * Comment out lines for tests not required.
 *
 * @author Owen Burroughs
 */

public class WSIFTestRunner {

    public static void main(String[] args) {
	    TestUtilities.startListeners();
        junit.textui.TestRunner.run(suite());
        TestUtilities.stopListeners();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite("All WSIF Tests");

		// Check all files for the dreaded ^M character! If you set
		// fix=on in the wsif.test.properties file, this test will
		// not only check the files but also fix them.
		//suite.addTest(new TestSuite(FileCharacterTest.class));

        // Test the initialisation of WSIF providers. This test needs to be
        // the FIRST of the functional tests as after this, WSIF will already
        // have been initialised.
        suite.addTest(new TestSuite(ProvidersInitialisationTest.class));

        //suite.addTest(new TestSuite(ShoppingCartTest.class));
        addIfAvaliable(suite, "ShoppingCartTest");
        suite.addTest(new TestSuite(AsyncTests.class));
        suite.addTest(new TestSuite(HeadersTest.class));
        suite.addTest(new TestSuite(FaultMsgTest.class));
        suite.addTest(new TestSuite(StockquoteTest.class));
        suite.addTest(new TestSuite(AddressBookTest.class));
        suite.addTest(new TestSuite(WsdlLoadingTest.class));
        suite.addTest(new TestSuite(InoutTest.class));
        suite.addTest(new TestSuite(DynamicInvokerTest.class));
        suite.addTest(new TestSuite(ZIPCodeTest.class));
        suite.addTest(new TestSuite(InteropTest.class));
        suite.addTest(new TestSuite(JNDIAddressBookTest.class));
        suite.addTest(new TestSuite(PlugableProvidersTest.class));
        suite.addTest(new TestSuite(InputPartsTest.class));
        suite.addTest(new TestSuite(OutputPartsTest.class));
        suite.addTest(new TestSuite(WildcardTest.class));
        suite.addTest(new TestSuite(ZipCodeAxisTest.class));
        suite.addTest(new TestSuite(NWBankTest.class));
        suite.addTest(new TestSuite(InteropDocTest.class));
        suite.addTest(new TestSuite(InteropDocWrappedTest.class));
        suite.addTest(new TestSuite(MessagingAttachmentsTest.class));
        suite.addTest(new TestSuite(SerializationTest.class));
        suite.addTest(new TestSuite(CharTest.class));
        suite.addTest(new TestSuite(SOAPENCTest.class));
        suite.addTest(new TestSuite(MissingInputPartTest.class));

        if (TestUtilities.areWeTesting("mime"))
            suite.addTest(new TestSuite(MimeTest.class));

        if (TestUtilities.areWeTesting("jms")) {
            suite.addTest(new TestSuite(JmsTest.class));
            suite.addTest(new TestSuite(JmsFaultTest.class));
        }

        return suite;
    }

    public static void addIfAvaliable(TestSuite suite, String className) {
	try {
	    Class klass = Class.forName("ShoppingCartTest");
	    suite.addTest(new TestSuite(klass));
	} catch(Exception ex) {
	}

    }
}
